#include <string>
#include <iostream>
#include <fstream>
#include <cstdlib>
#include <string>

#include <float.h>

#include "spline_io.h"

using namespace std;


/*=======================================================================
 * allocate/deallocate
 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
void 
alloc_spline_data(spline_coef& s_scoef)
{

  try
  {
    s_scoef.z_knots = new double[s_scoef.qz];
    s_scoef.t_knots = new double[s_scoef.num_theta_knots];
    s_scoef.c = new double[s_scoef.qz * s_scoef.num_theta_knots];
    s_scoef.cdifz = new double[(s_scoef.qz-1) * s_scoef.num_theta_knots];
    s_scoef.cdift = new double[s_scoef.qz * s_scoef.num_theta_knots];
  }

  catch ( ... )
  {
    cerr << "error allocating memory" << endl;
    exit(EXIT_FAILURE);
  }

}

/*-----------------------------------------------------------------------*/
void free_spline_data(spline_coef& s_scoef)
 {
  if ( NULL != s_scoef.c       ) { delete [] s_scoef.c       ; }
  if ( NULL != s_scoef.cdifz   ) { delete [] s_scoef.cdifz   ; }
  if ( NULL != s_scoef.cdift   ) { delete [] s_scoef.cdift   ; }
  if ( NULL != s_scoef.t_knots ) { delete [] s_scoef.t_knots ; }
  if ( NULL != s_scoef.z_knots ) { delete [] s_scoef.z_knots ; }
}


void
alloc_spline_coefs(spline_coef& s_scoef)
{
  alloc_spline_data(s_scoef );
}

/*-----------------------------------------------------------------------*/
void
free_spline_coefs( spline_coef& s_scoef )
{
  free_spline_data( s_scoef);
}

/*-----------------------------------------------------------------------*/
spline_coef
read_spline_data( std::string filename )
{
  spline_coef s_scoef;
  ifstream file( filename.c_str() );

  if ( ! file )
  {
    cerr << "unable to open spline coef file: " << filename << endl;
    exit(EXIT_FAILURE);
  }

  // throw exception on failed conversion (and eof) and
  // other bad things
  file.exceptions( ios::failbit | ios::badbit );

  try {
    
    file >> s_scoef.z_order;
    file >> s_scoef.qz;
    file >> s_scoef.zmin;
    file >> s_scoef.zmax;

    file >> s_scoef.theta_order;
    file >> s_scoef.num_theta_knots;
    file >> s_scoef.tmin;
    file >> s_scoef.tmax;


    s_scoef.num_z_knots = s_scoef.qz - s_scoef.z_order;


    alloc_spline_coefs( s_scoef );

    // read in spline knots

    for ( size_t i = 0 ; i < s_scoef.qz ; i++ )
      file >> s_scoef.z_knots[i];

    for ( size_t i = 0 ; i < s_scoef.num_theta_knots ; i++ )
      file >> s_scoef.t_knots[i];

    /*
     * read in spline data
     */
    double* c_data   = s_scoef.c;
    double* cz_data  = s_scoef.cdifz;
    double* ct_data  = s_scoef.cdift;

    for ( size_t jt = 0 ; jt < s_scoef.num_theta_knots ; ++jt )
    {
      for ( size_t i = 0 ; i < s_scoef.qz ; i++ )
	file >> c_data[i];
      c_data  += s_scoef.qz;
    }

    for ( size_t jt = 0 ; jt < s_scoef.num_theta_knots ; ++jt )
    {
      for ( size_t i = 0 ; i < s_scoef.qz-1 ; i++ )
	file >> cz_data[i];
      cz_data += s_scoef.qz - 1;
    }

    for ( size_t jt = 0 ; jt < s_scoef.num_theta_knots ; ++jt )
    {
      for ( size_t i = 0 ; i < s_scoef.qz ; i++ )
	file >> ct_data[i];

      ct_data += s_scoef.qz;
    }
  }

  catch (...) {

    if ( file.eof() )
    {
      cerr << filename << ": unexpected EOF" << endl;
      exit(EXIT_FAILURE);
    }

    else
    {
      cerr << filename << ": error converting input data at character " 
	   << file.tellg() << endl;
      exit(EXIT_FAILURE);
    }
  }

  return s_scoef;
}
